@@ -10,14 +10,22 @@ from shapely.geometry import shape
 import numpy as np
 
 from .io import read_features, Raster
-from .utils import (rasterize_geom, get_percentile, check_stats,
-                    remap_categories, key_assoc_val, boxify_points)
+from .utils import (
+    rasterize_geom,
+    get_percentile,
+    check_stats,
+    remap_categories,
+    key_assoc_val,
+    boxify_points,
+)
 
 
 def raster_stats(*args, **kwargs):
     """Deprecated. Use zonal_stats instead."""
-    warnings.warn("'raster_stats' is an alias to 'zonal_stats'"
-                  " and will disappear in 1.0", DeprecationWarning)
+    warnings.warn(
+        "'raster_stats' is an alias to 'zonal_stats'" " and will disappear in 1.0",
+        DeprecationWarning,
+    )
     return zonal_stats(*args, **kwargs)
 
 
@@ -33,21 +41,24 @@ def zonal_stats(*args, **kwargs):
 
 
 def gen_zonal_stats(
-        vectors, raster,
-        layer=0,
-        band=1,
-        nodata=None,
-        affine=None,
-        stats=None,
-        all_touched=False,
-        categorical=False,
-        category_map=None,
-        add_stats=None,
-        zone_func=None,
-        raster_out=False,
-        prefix=None,
-        geojson_out=False, 
-        boundless=True, **kwargs):
+    vectors,
+    raster,
+    layer=0,
+    band=1,
+    nodata=None,
+    affine=None,
+    stats=None,
+    all_touched=False,
+    categorical=False,
+    category_map=None,
+    add_stats=None,
+    zone_func=None,
+    raster_out=False,
+    prefix=None,
+    geojson_out=False,
+    boundless=True,
+    **kwargs,
+):
     """Zonal statistics of raster values aggregated to vector geometries.
 
     Parameters
@@ -113,11 +124,11 @@ def gen_zonal_stats(
         Original feature geometry and properties will be retained
         with zonal stats appended as additional properties.
         Use with `prefix` to ensure unique and meaningful property names.
-    
+
     boundless: boolean
         Allow features that extend beyond the raster dataset’s extent, default: True
         Cells outside dataset extents are treated as nodata.
-        
+
     Returns
     -------
     generator of dicts (if geojson_out is False)
@@ -130,20 +141,23 @@ def gen_zonal_stats(
     stats, run_count = check_stats(stats, categorical)
 
     # Handle 1.0 deprecations
-    transform = kwargs.get('transform')
+    transform = kwargs.get("transform")
     if transform:
-        warnings.warn("GDAL-style transforms will disappear in 1.0. "
-                      "Use affine=Affine.from_gdal(*transform) instead",
-                      DeprecationWarning)
+        warnings.warn(
+            "GDAL-style transforms will disappear in 1.0. "
+            "Use affine=Affine.from_gdal(*transform) instead",
+            DeprecationWarning,
+        )
         if not affine:
             affine = Affine.from_gdal(*transform)
 
-    cp = kwargs.get('copy_properties')
+    cp = kwargs.get("copy_properties")
     if cp:
-        warnings.warn("Use `geojson_out` to preserve feature properties",
-                      DeprecationWarning)
+        warnings.warn(
+            "Use `geojson_out` to preserve feature properties", DeprecationWarning
+        )
 
-    band_num = kwargs.get('band_num')
+    band_num = kwargs.get("band_num")
     if band_num:
         warnings.warn("Use `band` to specify band number", DeprecationWarning)
         band = band_num
@@ -151,9 +165,9 @@ def gen_zonal_stats(
     with Raster(raster, affine, nodata, band) as rast:
         features_iter = read_features(vectors, layer)
         for _, feat in enumerate(features_iter):
-            geom = shape(feat['geometry'])
+            geom = shape(feat["geometry"])
 
-            if 'Point' in geom.geom_type:
+            if "Point" in geom.geom_type:
                 geom = boxify_points(geom, rast)
 
             geom_bounds = tuple(geom.bounds)
@@ -164,35 +178,36 @@ def gen_zonal_stats(
             rv_array = rasterize_geom(geom, like=fsrc, all_touched=all_touched)
 
             # nodata mask
-            isnodata = (fsrc.array == fsrc.nodata)
+            isnodata = fsrc.array == fsrc.nodata
 
             # add nan mask (if necessary)
-            has_nan = (
-                np.issubdtype(fsrc.array.dtype, np.floating)
-                and np.isnan(fsrc.array.min()))
+            has_nan = np.issubdtype(fsrc.array.dtype, np.floating) and np.isnan(
+                fsrc.array.min()
+            )
             if has_nan:
-                isnodata = (isnodata | np.isnan(fsrc.array))
+                isnodata = isnodata | np.isnan(fsrc.array)
 
             # Mask the source data array
             # mask everything that is not a valid value or not within our geom
-            masked = np.ma.MaskedArray(
-                fsrc.array,
-                mask=(isnodata | ~rv_array))
+            masked = np.ma.MaskedArray(fsrc.array, mask=(isnodata | ~rv_array))
 
             # If we're on 64 bit platform and the array is an integer type
-            # make sure we cast to 64 bit to avoid overflow.
-            # workaround for https://github.com/numpy/numpy/issues/8433
-            if sys.maxsize > 2**32 and \
-                    masked.dtype != np.int64 and \
-                    issubclass(masked.dtype.type, np.integer):
-                masked = masked.astype(np.int64)
+            # make sure we cast to 64 bit to avoid overflow for certain numpy ops
+            if sys.maxsize > 2**32 and issubclass(masked.dtype.type, np.integer):
+                accum_dtype = "int64"
+            else:
+                accum_dtype = None  # numpy default
 
             # execute zone_func on masked zone ndarray
             if zone_func is not None:
                 if not callable(zone_func):
-                    raise TypeError(('zone_func must be a callable '
-                                     'which accepts function a '
-                                     'single `zone_array` arg.'))
+                    raise TypeError(
+                        (
+                            "zone_func must be a callable "
+                            "which accepts function a "
+                            "single `zone_array` arg."
+                        )
+                    )
                 value = zone_func(masked)
 
                 # check if zone_func has return statement
@@ -202,17 +217,22 @@ def gen_zonal_stats(
             if masked.compressed().size == 0:
                 # nothing here, fill with None and move on
                 feature_stats = dict([(stat, None) for stat in stats])
-                if 'count' in stats:  # special case, zero makes sense here
-                    feature_stats['count'] = 0
+                if "count" in stats:  # special case, zero makes sense here
+                    feature_stats["count"] = 0
             else:
                 if run_count:
                     keys, counts = np.unique(masked.compressed(), return_counts=True)
                     try:
-                        pixel_count = dict(zip([k.item() for k in keys],
-                                               [c.item() for c in counts]))
+                        pixel_count = dict(
+                            zip([k.item() for k in keys], [c.item() for c in counts])
+                        )
                     except AttributeError:
-                        pixel_count = dict(zip([np.asscalar(k) for k in keys],
-                                               [np.asscalar(c) for c in counts]))
+                        pixel_count = dict(
+                            zip(
+                                [np.asscalar(k) for k in keys],
+                                [np.asscalar(c) for c in counts],
+                            )
+                        )
 
                 if categorical:
                     feature_stats = dict(pixel_count)
@@ -221,63 +241,65 @@ def gen_zonal_stats(
                 else:
                     feature_stats = {}
 
-                if 'min' in stats:
-                    feature_stats['min'] = float(masked.min())
-                if 'max' in stats:
-                    feature_stats['max'] = float(masked.max())
-                if 'mean' in stats:
-                    feature_stats['mean'] = float(masked.mean())
-                if 'count' in stats:
-                    feature_stats['count'] = int(masked.count())
+                if "min" in stats:
+                    feature_stats["min"] = float(masked.min())
+                if "max" in stats:
+                    feature_stats["max"] = float(masked.max())
+                if "mean" in stats:
+                    feature_stats["mean"] = float(masked.mean(dtype=accum_dtype))
+                if "count" in stats:
+                    feature_stats["count"] = int(masked.count())
                 # optional
-                if 'sum' in stats:
-                    feature_stats['sum'] = float(masked.sum())
-                if 'std' in stats:
-                    feature_stats['std'] = float(masked.std())
-                if 'median' in stats:
-                    feature_stats['median'] = float(np.median(masked.compressed()))
-                if 'majority' in stats:
-                    feature_stats['majority'] = float(key_assoc_val(pixel_count, max))
-                if 'minority' in stats:
-                    feature_stats['minority'] = float(key_assoc_val(pixel_count, min))
-                if 'unique' in stats:
-                    feature_stats['unique'] = len(list(pixel_count.keys()))
-                if 'range' in stats:
+                if "sum" in stats:
+                    feature_stats["sum"] = float(masked.sum(dtype=accum_dtype))
+                if "std" in stats:
+                    feature_stats["std"] = float(masked.std())
+                if "median" in stats:
+                    feature_stats["median"] = float(np.median(masked.compressed()))
+                if "majority" in stats:
+                    feature_stats["majority"] = float(key_assoc_val(pixel_count, max))
+                if "minority" in stats:
+                    feature_stats["minority"] = float(key_assoc_val(pixel_count, min))
+                if "unique" in stats:
+                    feature_stats["unique"] = len(list(pixel_count.keys()))
+                if "range" in stats:
                     try:
-                        rmin = feature_stats['min']
+                        rmin = feature_stats["min"]
                     except KeyError:
                         rmin = float(masked.min())
                     try:
-                        rmax = feature_stats['max']
+                        rmax = feature_stats["max"]
                     except KeyError:
                         rmax = float(masked.max())
-                    feature_stats['range'] = rmax - rmin
+                    feature_stats["range"] = rmax - rmin
 
-                for pctile in [s for s in stats if s.startswith('percentile_')]:
+                for pctile in [s for s in stats if s.startswith("percentile_")]:
                     q = get_percentile(pctile)
                     pctarr = masked.compressed()
                     feature_stats[pctile] = np.percentile(pctarr, q)
 
-            if 'nodata' in stats or 'nan' in stats:
+            if "nodata" in stats or "nan" in stats:
                 featmasked = np.ma.MaskedArray(fsrc.array, mask=(~rv_array))
 
-                if 'nodata' in stats:
-                    feature_stats['nodata'] = float((featmasked == fsrc.nodata).sum())
-                if 'nan' in stats:
-                    feature_stats['nan'] = float(np.isnan(featmasked).sum()) if has_nan else 0
+                if "nodata" in stats:
+                    feature_stats["nodata"] = float((featmasked == fsrc.nodata).sum())
+                if "nan" in stats:
+                    feature_stats["nan"] = (
+                        float(np.isnan(featmasked).sum()) if has_nan else 0
+                    )
 
             if add_stats is not None:
                 for stat_name, stat_func in add_stats.items():
                     try:
-                        feature_stats[stat_name] = stat_func(masked, feat['properties'])
+                        feature_stats[stat_name] = stat_func(masked, feat["properties"])
                     except TypeError:
                         # backwards compatible with single-argument function
                         feature_stats[stat_name] = stat_func(masked)
 
             if raster_out:
-                feature_stats['mini_raster_array'] = masked
-                feature_stats['mini_raster_affine'] = fsrc.affine
-                feature_stats['mini_raster_nodata'] = fsrc.nodata
+                feature_stats["mini_raster_array"] = masked
+                feature_stats["mini_raster_affine"] = fsrc.affine
+                feature_stats["mini_raster_nodata"] = fsrc.nodata
 
             if prefix is not None:
                 prefixed_feature_stats = {}
@@ -288,9 +310,9 @@ def gen_zonal_stats(
 
             if geojson_out:
                 for key, val in feature_stats.items():
-                    if 'properties' not in feat:
-                        feat['properties'] = {}
-                    feat['properties'][key] = val
+                    if "properties" not in feat:
+                        feat["properties"] = {}
+                    feat["properties"][key] = val
                 yield feat
             else:
                 yield feature_stats
